#!/usr/bin/env python3
# -*- coding: utf8 -*-

import os
from pathlib import Path
from typing import List
from config import config

from students import Students
from exam import Exam
from pdftools import PDFTools
from metadata import FileMetaData

#print(config.tao.host)

# File format: Nom;DateNaissance;SCEI 
# File encoding: cp1252
CSV_FILE = 'epreuve-E4_2021-03-31_ADA_ADB_AM.csv'
CSV_DIR = '../sample_data'
base_filename = os.path.splitext(CSV_FILE)[0]

metadata = FileMetaData.from_filename(base_filename)
students = Students.from_csv(CSV_DIR + '/' + CSV_FILE, detectDelimiter=True)
students.generate_passwords()


# exam = Exam(title='Anglais', date='31 Mars 2021', timeslot='10:00 - 10:30', duration='30 min')
# student = Student(fullname='HARTMANN MOSCISKI Quinten', birth_date='1978-03-19', scei='54642', password='tbandtshe')

# File encoding: utf8
OUTPUT_DIR = '../output'
Path(OUTPUT_DIR).mkdir(parents=False, exist_ok=True)

students.as_json(f'{OUTPUT_DIR}/{base_filename}.json')

exam = Exam.from_metadata(metadata)
html_content = students.coupons_as_html(exam)

# save as .html
with open(f'{OUTPUT_DIR}/{base_filename}.html', 'w') as output_html_file:
    output_html_file.write(html_content)

# convert as .pdf
PDFTools.from_html(html_content, f'{OUTPUT_DIR}/{base_filename}.pdf')
