import os
import re
from pathlib import Path
from dataclasses import dataclass
from typing import List
from enum import Enum
import pendulum

class MetaDataTestEnum(Enum):
    ATTENDANCE = 'E0'
    DIGITAL = 'E1'
    ENGLISH = 'E4'

class MetaDataPeriodEnum(Enum):
    AM = 'AM'
    PM = 'PM'

@dataclass
class FileMetaData:
    test: MetaDataTestEnum  # 'E0' -> presence | 'E1' -> QCM numerique | 'E4' -> QCM anglais
    date: str # YYYY-MM-DD
    period: MetaDataPeriodEnum # 'AM' | 'PM'

    @classmethod
    def from_filename(cls, filename : str):
        basename = os.path.basename(filename)
        name = os.path.splitext(basename)[0]

        # 'epreuve-E0_2019-06-28_ADA_ADB_AM'
        m = re.match(r"epreuve-(E[014])_(\d\d\d\d-\d\d-\d\d)_ADA_ADB_(AM|PM)", name)
        if not m:
            raise Exception(f'Invalid filename format (expecting: <epreuve-E?_YYYY-MM-DD_ADA_ADB_?M> but got <{name}>)')
        else:
            test = MetaDataTestEnum(m[1])
            date = pendulum.from_format(m[2], 'YYYY-MM-DD')
            period = MetaDataPeriodEnum(m[3])

            return FileMetaData(test, date, period)
